/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/

#include "derivative.h" /* include peripheral declarations S32K144 */
#include "headerMain.h"
#include "PT2001_irq.h"
#include "PT2001_spi_map.h"

//========================================================================================
// Global Variables
//========================================================================================
int result = 0;
volatile char states = 0;
volatile int flag_IRQB = 0;
volatile int flag_EOI = 0;
int count = 0;
int drv_int_counter = 0;
const int sw_retry = 5;
uint16_t rxData = 0;

const int auto_retry = 2;
uint16_t cksys_missing_error = 0;
uint16_t cksys_missing = 0;
uint16_t interrupt_reg = 0;
uint16_t drv_irq = 0;
uint16_t checksum_irq =0;
uint16_t drv_status = 0;
uint16_t spi_irq = 0;
volatile int seq_counter = 0;
int iprof_counter = 0;
uint16_t auto_irq = 0;
uint16_t sw_irq = 0;
uint16_t inj1_error = 0;
uint16_t inj2_error = 0;
uint16_t inj3_error = 0;
uint16_t inj4_error = 0;
uint16_t inj5_error = 0;
uint16_t inj6_error = 0;
uint16_t fp_error = 0;
uint16_t dcdc_error = 0;
volatile bool flag_start1_fail = 0;
volatile bool flag_start2_fail = 0;
volatile bool flag_start3_fail = 0;
volatile bool flag_start4_fail = 0;
volatile bool flag_start5_fail = 0;
volatile bool flag_start6_fail = 0;

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : main
// Description     : Program entry point. Programs the PT2001 and runs an injection
//                   sequence.
// Return type     : int
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
int main(void)
{
	// Initialize the hardware
 	init_CLOCK();
	init_GPIO();
	init_SPI();
	init_ADC();
	InputCaptureTimer_init();
	PWM_1MHz();

	// Reset the PT2001 device
	SET_RESETB_LOW;
	delay(1);
	SET_RESETB_HIGH;
	delay100us(1);                                 // Wait for the PLL time to lock

	if (ID_Check () == 0) { return -1; };          // Check if SPI works and that device is the good one
	if (CLK_check() == 0) { return -1; };          // Check if 1MHz CLK is properly connected
	if (DRVEN_check() == 0) { return -1; };        // Check if DRVEN monitoring works using SPI register 1B2h
	if (BIST_check(1) == 0) { return -1; };        // Run BIST
	if (OA_path_check(1,1) == 0) { return -1; };   // Check OA pin connection to MCU ADC

	// Program the device
	ProgramDevice();

	if (Checksum_check() == 0) { return -1; };     // Checksum pass or fail
	if (Driver_Status_Init() == 0) { return -1; }; // Check if all regulators are at ON
	//if (Bootstrap_check() == 0) { return -1; };    // Check if bootstrap caps are charged (should be done after 35ms, need load on all EVB connectors)

	// Enable interrupts
	INT_SYS_EnableIRQ(PORTD_IRQn);
	INT_SYS_EnableIRQ(PORTC_IRQn);
	INT_SYS_EnableIRQ(FTM1_IRQn);

	// Initialize port A interrupt
	PORT_HAL_ClearPinIntFlagCmd(PORTA, 3);
	INT_SYS_ClearPending(PORTA_IRQn);
	INT_SYS_EnableIRQ(PORTA_IRQn);

#ifdef PT2001    // This function is not on MC33816
	if (Read_VbatADC () < 6) { return -1; }; // Check if SPI works and that device is the good one
#endif

	SET_DRVEN_HIGH;
	delay(100);						// Need to wait before sending start pulse to let DCDC reach 65V

	for (;;)
	{
		// sequences 1, 2
		switch (states)
		{
			case sequence_1:
			{

				// Read ADC potentiometer
				result = read_ADC(ADC_INPUTCHAN_AD12);
				send_single_SPI_Cmd(WRITE, ch1_ctrl_reg_uc0, 0x0);  // Init Control reg to set current profile by default

				if (seq_counter == 1)
				{
					InjectorSequence1(result);
				}
				else
				{
					Off_sequence();
				}
				break;
			}
			case sequence_2:
			{
				// Read ADC potentiometer
				result = read_ADC(ADC_INPUTCHAN_AD12);

				// Toggle Start Pins
				InjectorSequence1(result);

				switch (iprof_counter)
				{
					case 0:
					{
						send_single_SPI_Cmd(WRITE, ch1_ctrl_reg_uc0, 0x0);
						break;
					}
					case 1:
					{
						send_single_SPI_Cmd(WRITE, ch1_ctrl_reg_uc0, 0x8000);
						break;
					}
					case 2:
					{
						send_single_SPI_Cmd(WRITE, ch1_ctrl_reg_uc0, 0xC000);
						break;
					}

					case 3:
					{
						send_single_SPI_Cmd(WRITE, ch1_ctrl_reg_uc0, 0x4000);
						break;
					}  // end SPI seq with different profile current
					break;
				}
				break;
			}
		} // END of switch case

		//========================================================================================
		// START OF IRQB CHECK
		//========================================================================================
		if (flag_IRQB == 1)
		{
			flag_IRQB = 0;
			// Read interrupt register
			interrupt_reg = send_single_SPI_Cmd(READ, main_Interrupt_register, 0x00);  // This latches only the first interrupt and only gets cleared when all interrupt are cleared

			if (interrupt_reg != 0)
			{
				// Driver disable IRQ
				drv_irq = interrupt_reg & 0x1F00;
				if (drv_irq != 0) // One of the halt bits is at 1
				{
					ProcessDriverInterrupts();
				}

				// Automatic IRQ (halt bits)
				auto_irq = interrupt_reg & 0x000F;
				if (auto_irq != 0) // One of the halt bits is at 1
				{
					ProcessAutomaticInterrupts();
					ProcessSoftwareInterrupts(); // Needed in case a SW interrupt also happened
				}

				// Software IRQ (irq bits)
				sw_irq = interrupt_reg & 0x00F0;
				if (sw_irq != 0) // One of the irq bits is at 1
				{
					ProcessSoftwareInterrupts();
					ProcessAutomaticInterrupts();
				}
			} // End of Interrupt auto SW interrupt
		} // End of IRQB flag
	} // End of for loop (main)

	return 0;
} // end of main
