/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * adc.c
 *
 *  Created on: Apr 12, 2016
 *      Author: B52321
 */

#include "adc.h"

adc_chan_config_t channel_conf =  //ADC_INPUTCHAN_AD14 ADC_INPUTCHAN_AD6 ADC_INPUTCHAN_AD7
{
	.channel = ADC_INPUTCHAN_AD12,
	.interruptEnable = false,
};

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : init_ADC
// Description     : Initializes the ADC subsystem.
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void init_ADC(void)
{
	adc_converter_config_t adc_config =
	{
		.clockDivide = ADC_CLK_DIVIDE_8,
		.sampleTime = 255U,
		.resolution = ADC_RESOLUTION_8BIT,
		.inputClock = ADC_CLK_ALT_1,
		.trigger = ADC_TRIGGER_SOFTWARE,
		.dmaEnable = false,
		.voltageRef = ADC_VOLTAGEREF_VREF, //ADC_VOLTAGEREF_VALT,
		.continuousConvEnable = false,
	};

	ADC_DRV_ConfigConverter(0U, &adc_config);
	ADC_DRV_ConfigConverter(1U, &adc_config);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : read_ADC
// Description     : Read from ADC module 0
// Return type     : uint16_t 
// Argument        : unsigned char channel
//////////////////////////////////////////////////////////////////////////////////////////
uint16_t read_ADC(unsigned char channel)
{
	uint16_t result;
	channel_conf.channel = channel;
	/* Configure ADC channel and software trigger a conversion */
	ADC_DRV_ConfigChan(0U, 0, &channel_conf);
	/* Wait for the conversion to be done */
	ADC_DRV_WaitConvDone(0U);
	/* Store the channel result into a local variable */
	ADC_DRV_GetChanResult(0U, 0U, &result);
	return result;
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : read_ADC1
// Description     : Read from ADC module 1
// Return type     : uint16_t 
// Argument        : unsigned char channel
//////////////////////////////////////////////////////////////////////////////////////////
uint16_t read_ADC1(unsigned char channel)
{
	uint16_t result;
	channel_conf.channel = channel;
	/* Configure ADC channel and software trigger a conversion */
	ADC_DRV_ConfigChan(1U, 0, &channel_conf);
	/* Wait for the conversion to be done */
	ADC_DRV_WaitConvDone(1U);
	/* Store the channel result into a local variable */
	ADC_DRV_GetChanResult(1U, 0U, &result);
	return result;
}
