/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * clk.c
 *
 *  Created on: Apr 11, 2016
 *      Author: B52321
 */
#include "fsl_clock_manager.h"
#include "clk.h"

#define FSL_CLOCK_MANAGER_CONFIG_CNT (1)
#define FSL_CLOCK_MANAGER_CALLBACK_CNT (0)


peripheral_clock_config_t peripheralClockConfig0[11] =
{
    {
        .clockName = PCC_PORTD_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_OFF,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_PORTC_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_OFF,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_PORTA_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_OFF,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_PORTE_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_OFF,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_PORTB_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_SOSC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_LPSPI0_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_FIRC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_SCG0_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_SOSC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_ADC0_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_SOSC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
	{
		.clockName        = PCC_ADC1_CLOCK,
		.clkGate          = true,
		.clkSrc           = CLK_SRC_SOSC,
		.frac             = MULTIPLY_BY_ONE,
		.divider          = DIVIDE_BY_ONE,
	},

    {
        .clockName = PCC_FLEXTMR2_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_SOSC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
    {
        .clockName = PCC_FLEXTMR1_CLOCK,
        .clkGate = true,
        .clkSrc = CLK_SRC_SOSC,
        .frac = MULTIPLY_BY_ONE,
        .divider = DIVIDE_BY_ONE,
    },
};

clock_manager_user_config_t clockMan1_InitConfig0 =
{
    /*! @brief Configuration of SIRC */
    .scgConfig = {
        .sircConfig =
        {
            .range = SCG_SIRC_RANGE_HIGH,
            .div1 = 2,
            .div2 = 2,
            .initialize = true,            /*!< Initialize or not the SIRC module.        */
            .enableInStop = false,         /*!< SIRC is enable or not in stop mode.       */
            .enableInLowPower = false,
        },
        .fircConfig =
        {
            .range = SCG_FIRC_RANGE_48M,   /*!< Fast IRC is trimmed to 48MHz.  */
            .div1 = 1,                     /*!< Divider for platform asynchronous clock.  */
            .div2 = 1,                     /*!< Divider for bus asynchronous clock.       */
            .enableInStop = false,         /*!< FIRC is enable or not in stop mode.       */
            .enableInLowPower = false,     /*!< FIRC is enable or not in lowpower mode.   */
            .regulator = false,            /*!< FIRC regulator is enable or not.          */
            .locked = false,               /*!< FIRC Control Register can be written.     */
            .enableTrim = false,           /*!< FIRC is trimmed to an external clock.     */
            .updateTrim = false,           /*!< Enable FIRC trimming updates.             */
            .initialize = true,
        },
        .soscConfig =
        {
            .freq = 20000000, //8MHz       /*!< System OSC frequency.                                */
            .monitorMode = 0,              /*!< System OSC Clock monitor mode.                       */
            .extRef = SCG_SOSC_REF_OSC,    /*!< System OSC External Reference Select.                */
            .gain = 0,                     /*!< System OSC high-gain operation.                      */
            .range = SCG_SOSC_RANGE_HIGH,  /*!< High frequency range selected for the crystal OSC (8 Mhz to 32 Mhz).   */
            .div1 = 1,                     /*!< Divider for platform asynchronous clock.             */
            .div2 = 1,                     /*!< Divider for bus asynchronous clock.                  */
            .enableInStop = false,         /*!< System OSC is enable or not in stop mode.            */
            .enableInLowPower = false,     /*!< System OSC is enable or not in low power mode.       */
            .enableErClk = false,          /*!< System OSC 3V ERCLK output clock.                    */
            .locked = false,               /*!< System OSC Control Register can be written.          */
            .initialize = true,            /*!< Initialize or not the System OSC module.             */
        },
        .spllConfig =
        {
            .monitorMode = 0,              /*!< Clock monitor mode selected.                    */
            .prediv = 0,                   /*!< PLL reference clock divider.                    */
            .mult = 0,                     /*!< System PLL multiplier.                          */
            .div1 = 2,                     /*!< Divider for platform asynchronous clock.        */
            .div2 = 2,                     /*!< Divider for bus asynchronous clock.             */
            .enableInStop = false,         /*!< System PLL clock is enable or not in stop mode. */
            .locked = false,               /*!< System PLL Control Register can be written.     */
            .initialize = true,
        },
        .clockModeConfig = 
        {
            .rccrConfig =         /*!< Run Clock Control configuration.                 */
            {
                .divSlow = SCG_SYSTEM_CLOCK_DIV_BY_2,
                .divBus = SCG_SYSTEM_CLOCK_DIV_BY_2,
                .divPlat = SCG_SYSTEM_CLOCK_DIV_BY_1,
                .divCore = SCG_SYSTEM_CLOCK_DIV_BY_2,
                .src = SCG_SYSTEM_CLOCK_DIV_BY_4,
            },
            .vccrConfig =         /*!< VLPR Clock Control configuration.                */
            {
                .divSlow = 2,
                .divBus = 0,
                .divPlat = 0,
                .divCore = 0,
                .src = 1,
            },
            .hccrConfig =         /*!< HSRUN Clock Control configuration.               */
            {
                .divSlow = 2,
                .divBus = 0,
                .divPlat = 0,
                .divCore = 0,
                .src = 1,
            },
            .alternateClock = 3,  /*!< Alternate clock used during initialization       */
            .initialize = true,   /*!< Initialize or not the Clock Mode Configuration.  */
        },

    },
    .simConfig =
    {
        .clockOutConfig = 
        {
            .initialize = true,   /*!< Initialize or not the ClockOut clock.  */
            .source = 0,          /*!< ClockOut source select. */
        },
    },
    .pccConfig =
    {
        .count = 11,                                 /*!< Number of peripherals to be configured.               */
        .peripheralClocks = peripheralClockConfig0,  /*!< Pointer to the peripheral clock configurations array. */
    },
};

/*! @brief Array of pointers to User configuration structures */
clock_manager_user_config_t const * g_clockManConfigsArr[] =
{
    &clockMan1_InitConfig0
};

void init_CLOCK(void)
{
    clock_manager_callback_user_config_t * g_clockManCallbacksArr[] = { (void*)0 };

    CLOCK_SYS_Init(g_clockManConfigsArr, FSL_CLOCK_MANAGER_CONFIG_CNT,
        g_clockManCallbacksArr, FSL_CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_FORCIBLE);

    SCG->FIRCDIV = SCG_FIRCDIV_FIRCDIV1(0b01)  /*FIRC DIV1=1*/
        | SCG_FIRCDIV_FIRCDIV2(0b01);          /*FIRC DIV2=1*/
}
