/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * functions.c
 *
 *  Created on: Apr 12, 2016
 *      Author: B52321
 */

#include "lpspi.h"
#include "functions.h"
#include "PT2001_spi_map.h"

const int DELAY_FACTOR = 1599;                // This needs to be set for the specific MCU being used
const int DELAY_FACTOR2 = 159;
extern volatile bool flag_start1_fail;
extern volatile bool flag_start2_fail;
extern volatile bool flag_start3_fail;
extern volatile bool flag_start4_fail;
extern volatile bool flag_start5_fail;


//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : delay
// Description     : Delays for the specified number of milliseconds.
// Return type     : void 
// Argument        : unsigned long msDelay
//////////////////////////////////////////////////////////////////////////////////////////
void delay(unsigned long msDelay)    // RICARDO HELP TO GET SOMETHING ACCURATE
{
	int i = 0;

	for (i = 0; i < (msDelay * DELAY_FACTOR); i++);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : delay100us
// Description     : Delays for the specified number of 100us increments.
// Return type     : void 
// Argument        : unsigned long delay
//////////////////////////////////////////////////////////////////////////////////////////
void delay100us(unsigned long delay)  // RICARDO HELP TO GET SOMETHING ACCURATE IN 1us or 10us not 100us
{
	int i = 0;

	for (i = 0; i < (delay * DELAY_FACTOR2); i++);
}


// Delay constants (ms)
unsigned long tOff = 250;
const unsigned long tINJ = 2;
const unsigned long tFP = 4;
const unsigned long tPause = 2000;

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : InjectorSequence1
// Description     : Toggles all injectors and fuel pump.
// Return type     : void
// Argument        : unsigned char loopDelay
//////////////////////////////////////////////////////////////////////////////////////////
void InjectorSequence1(unsigned char loopDelay)
{
	tOff = loopDelay + 50;

	if (flag_start1_fail == 0)
	{
		// Toggle Injector 1
		SET_START1_HIGH;
		delay(tINJ);
		SET_START1_LOW;
		delay(tOff);
	}

	if (flag_start5_fail == 0)
	{
		// Toggle Fuel Pump
		SET_START5_HIGH;
		delay(tFP);
		SET_START5_LOW;
		delay(tOff);
	}

	if (flag_start3_fail == 0)
	{
		// Toggle Injector 3
		SET_START3_HIGH;
		delay(tINJ);
		SET_START3_LOW;
		delay(tOff);
	}

	if (flag_start5_fail == 0)
	{
		// Toggle Fuel Pump
		SET_START5_HIGH;
		delay(tFP);
		SET_START5_LOW;
		delay(tOff);
	}

	if (flag_start4_fail == 0)
	{
		// Toggle Injector 4
		SET_START4_HIGH;
		delay(tINJ);
		SET_START4_LOW;
		delay(tOff);
	}

	if (flag_start5_fail == 0)
	{
		// Toggle Fuel Pump
		SET_START5_HIGH;
		delay(tFP);
		SET_START5_LOW;
		delay(tOff);
	}

	if (flag_start2_fail == 0)
	{
		// Toggle Injector 2
		SET_START2_HIGH;
		delay(tINJ);
		SET_START2_LOW;
		delay(tOff);
	}

	if (flag_start5_fail == 0)
	{
		// Toggle Fuel Pump
		SET_START5_HIGH;
		delay(tFP);
		SET_START5_LOW;
		delay(tOff);
	}

	delay(loopDelay + 200);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : OFF_sequence
// Description     :
// Return type     : void
// Argument        : unsigned char loopDelay
//////////////////////////////////////////////////////////////////////////////////////////
void Off_sequence(void)
{
		SET_START1_LOW;
		SET_START2_LOW;
		SET_START3_LOW;
		SET_START4_LOW;
		SET_START5_LOW;
		SET_START6_LOW;
}
