/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * gpio.c
 *
 *  Created on: Apr 11, 2016
 *      Author: B52321
 */
#include "fsl_gpio_hal.h"
#include "fsl_pins_driver.h"
#include "gpio.h"

//===============================================================================
// Macros for configuring port pins
//===============================================================================
#define ConfigureGPIO(PORT, PIN, PORT_GPIO_BASE, DIRECTION) \
{ \
    .base          = PORT, \
    .pinPortIdx    = PIN,   \
    .pullConfig    = PORT_INTERNAL_PULL_NOT_ENABLED, \
    .passiveFilter = false, \
    .driveSelect   = PORT_LOW_DRIVE_STRENGTH, \
    .mux           = PORT_MUX_AS_GPIO, \
    .pinLock       = false, \
    .intConfig     = PORT_DMA_INT_DISABLED, \
    .clearIntFlag  = false, \
    .gpioBase      = PORT_GPIO_BASE, \
    .direction     = DIRECTION, }

#define ConfigurePort(PORT, PIN, MUX, INT_CONFIG, PORT_GPIO_BASE, DIRECTION) \
{ \
    .base          = PORT, \
    .pinPortIdx    = PIN,   \
    .pullConfig    = PORT_INTERNAL_PULL_NOT_ENABLED, \
    .passiveFilter = false, \
    .driveSelect   = PORT_LOW_DRIVE_STRENGTH, \
    .mux           = MUX, \
    .pinLock       = false, \
    .intConfig     = INT_CONFIG, \
    .clearIntFlag  = false, \
    .gpioBase      = PORT_GPIO_BASE, \
    .direction     = DIRECTION, }


//===============================================================================
// Initialize pins - See PinSettings component for more info
//===============================================================================
pin_settings_config_t g_pin_mux_InitConfigArr[] =
{
    // LED on KITK144 will be use as green when no error
    // and RED when error (like IRQB pulled low)
    ConfigureGPIO(PORTD, 16, PTD, GPIO_OUTPUT_DIRECTION),  // LED GREEN
    ConfigureGPIO(PORTD, 15, PTD, GPIO_OUTPUT_DIRECTION),  // LED RED
    ConfigureGPIO(PORTD, 0, PTD, GPIO_INPUT_DIRECTION),    // LED BLUE  //NOT USED SO DEFINED AS INPUT

    // PT2001 SPI Bus Definition PTB2 3 4 5 are used
    ConfigurePort(PORTB, 2, PORT_MUX_ALT3, PORT_DMA_INT_DISABLED, PTB, GPIO_OUTPUT_DIRECTION),  // SPI SCLK
    ConfigurePort(PORTB, 3, PORT_MUX_ALT3, PORT_DMA_INT_DISABLED, PTB, GPIO_OUTPUT_DIRECTION),  // SPI MISO
    ConfigurePort(PORTB, 4, PORT_MUX_ALT3, PORT_DMA_INT_DISABLED, PTB, GPIO_INPUT_DIRECTION),   // SPI MOSI
    ConfigurePort(PORTB, 5, PORT_MUX_ALT3, PORT_DMA_INT_DISABLED, PTB, GPIO_OUTPUT_DIRECTION),  // SPI CSB

    // START Pin Definition START1 to 7 are used as outputS and Start8
    // as an input since it is a multiplexed pin on the KITPT2001
    ConfigureGPIO(PORTB, 8, PTB, GPIO_OUTPUT_DIRECTION),   // START1
    ConfigureGPIO(PORTB, 9, PTB, GPIO_OUTPUT_DIRECTION),   // START2
    ConfigureGPIO(PORTB, 10, PTB, GPIO_OUTPUT_DIRECTION),  // START3
    ConfigureGPIO(PORTB, 11, PTB, GPIO_OUTPUT_DIRECTION),  // START4
    ConfigureGPIO(PORTC, 10, PTC, GPIO_OUTPUT_DIRECTION),  // START5
    ConfigureGPIO(PORTC, 11, PTC, GPIO_OUTPUT_DIRECTION),  // START6
    ConfigureGPIO(PORTD, 13, PTD, GPIO_OUTPUT_DIRECTION),  // START7
    ConfigureGPIO(PORTA, 8, PTA, GPIO_INPUT_DIRECTION),    // START8 (SET AS INPUT)

    // Definition of all FLAG pins from the PT2001, all of them are
    // used as inputs in this example but they can also be defined as outputs
    ConfigureGPIO(PORTD, 9, PTD, GPIO_INPUT_DIRECTION),    // FLAG0
    //ConfigureGPIO(PORTD,  8, PTD, GPIO_INPUT_DIRECTION),   // FLAG1 USED as interrupt for EOI detection
    //ConfigureGPIO(PORTC,  8, PTC, GPIO_INPUT_DIRECTION),   // FLAG2
    ConfigureGPIO(PORTC, 9, PTC, GPIO_INPUT_DIRECTION),    // FLAG3
    ConfigureGPIO(PORTD, 17, PTD, GPIO_INPUT_DIRECTION),   // FLAG4

    //DRVEN pin
    // Definition of DRVEN pin, two port are used for DRVEN since it is our safety pin.
    // One port is use as input and the other as output. This pin if pulled low will shutdown all injection
    ConfigureGPIO(PORTE, 1, PTE, GPIO_INPUT_DIRECTION),   // DRVEN READ  (need to be set as interrupt, if pulled low a check needs to be done)
    ConfigureGPIO(PORTE, 10, PTE, GPIO_INPUT_DIRECTION),  // DRVEN READ  (for 2nd revision of the S32K144 Freedom board)
    ConfigureGPIO(PORTA, 9, PTA, GPIO_OUTPUT_DIRECTION),  // DRVEN WRITE

    // Debug pin DBG is used for several purpose:
    //    - If pulled low during startup: VCCP internal will not start until enable by SPI
    //    - If pulled high during startup: VCCP internal regulator will start to charge bootstrap capacitor
    //    - If defined as input the DBG will allow a external connection of the KITPSCDEBUG Tracer for debug
    // So in order to make it compliant with most application we will define it as an input, and thanks to
    // PT2001 weak pull up on this pin the VCCP int regulator will start to charge cap.*/


    // DBG
    //IMPORTANT THIS PIN IS SHARED WITH SBC INT Pin (need to remove one resistance if used, will be fixed in next K144 released)
    ConfigureGPIO(PORTE, 12, PTE, GPIO_INPUT_DIRECTION),   // DBG

    // INT CLK 1MHz
    // INT CLK 1MHz, this pin will send a 1MHz CLK to PT2001, in case of failure
    // PT2001 will switch to backup CLK PIN TO be defined as a PWM
    {
        .base = PORTD,  //1MHz CLK
        .pinPortIdx = 14u,
        .pullConfig = PORT_INTERNAL_PULL_UP_ENABLED,
        .passiveFilter = false,
        .driveSelect = PORT_LOW_DRIVE_STRENGTH,
        .mux = PORT_MUX_ALT2,
        .pinLock = false,
        .intConfig = PORT_DMA_INT_DISABLED,
        .clearIntFlag = true,
        .gpioBase = PTD,
        .direction = GPIO_OUTPUT_DIRECTION,
    },

    // INTERRUPT PINS
    // IRQB pin
    // IRQB pin is the PT2001 interrupt pin. Pulled low in case of issue.
    // Defined as input interrupt
    {
        .base = PORTA,  //IRQB
        .pinPortIdx = 3u,
        .pullConfig = PORT_INTERNAL_PULL_NOT_ENABLED,
        .passiveFilter = false,
        .driveSelect = PORT_LOW_DRIVE_STRENGTH,
        .mux = PORT_MUX_AS_GPIO,
        .pinLock = false,
        .intConfig = PORT_INT_FALLING_EDGE,
        .clearIntFlag = false,
        .gpioBase = PTA,
        .direction = GPIO_INPUT_DIRECTION,
    },

    //FLAG1
    // Flag1 is used to let the MCU know that the EOI is finished
    {
        .base = PORTD,   //FLAG1
        .pinPortIdx = 8u,
        .pullConfig = PORT_INTERNAL_PULL_DOWN_ENABLED,
        .passiveFilter = false,
        .driveSelect = PORT_LOW_DRIVE_STRENGTH,
        .mux = PORT_MUX_AS_GPIO, //@ROD input capture FTM
        .pinLock = false,
        .intConfig = PORT_INT_EITHER_EDGE,
        .clearIntFlag = false,
        .gpioBase = PTD,
        .direction = GPIO_INPUT_DIRECTION,
    },

    // Button interrupt ports for controlling sequences
    {
        .base = PORTC,   // BTN SW2
        .pinPortIdx = 12u,
        .pullConfig = PORT_INTERNAL_PULL_NOT_ENABLED,
        .passiveFilter = false,
        .driveSelect = PORT_LOW_DRIVE_STRENGTH,
        .mux = PORT_MUX_AS_GPIO,
        .pinLock = false,
        .intConfig = PORT_INT_FALLING_EDGE,
        .clearIntFlag = false,
        .gpioBase = PTC,
        .direction = GPIO_INPUT_DIRECTION,
    },

    {
        .base = PORTC,   // BTN SW3
        .pinPortIdx = 13u,
        .pullConfig = PORT_INTERNAL_PULL_NOT_ENABLED,
        .passiveFilter = false,
        .driveSelect = PORT_LOW_DRIVE_STRENGTH,
        .mux = PORT_MUX_AS_GPIO,
        .pinLock = false,
        .intConfig = PORT_INT_FALLING_EDGE,
        .clearIntFlag = false,
        .gpioBase = PTC,
        .direction = GPIO_INPUT_DIRECTION,
    },

    // RESETB pin is PT2001 reset input it is active High
    ConfigureGPIO(PORTA, 2, PTA, GPIO_OUTPUT_DIRECTION),  // RESETB

    // Potentiometer on KITK144 use to control RPM (ADC)   // TO BE UPDATED
    ConfigurePort(PORTC, 14, PORT_PIN_DISABLED, PORT_DMA_INT_DISABLED, PTD, GPIO_OUTPUT_DIRECTION),  // ADC POTENTIOMETER

    // OA output definition (ADC), they will be used to read VCC2p5 values and check current profile or EOI
    ConfigurePort(PORTD, 4, PORT_PIN_DISABLED, PORT_DMA_INT_DISABLED, PTD, GPIO_OUTPUT_DIRECTION),   // ADC OA3
    ConfigurePort(PORTB, 12, PORT_PIN_DISABLED, PORT_DMA_INT_DISABLED, PTB, GPIO_OUTPUT_DIRECTION),  // ADC OA2
    ConfigurePort(PORTB, 0, PORT_PIN_DISABLED, PORT_DMA_INT_DISABLED, PTB, GPIO_OUTPUT_DIRECTION),   // ADC OA1

    //===============================================================================
    // These ports will only be used for the demo with LED, not with normal EVB
    //===============================================================================

    ConfigureGPIO(PORTB, 1, PTB, GPIO_OUTPUT_DIRECTION),   // LED1 driver
    ConfigureGPIO(PORTA, 6, PTA, GPIO_OUTPUT_DIRECTION),   // LED2 driver
    ConfigureGPIO(PORTC, 0, PTC, GPIO_OUTPUT_DIRECTION),   // LED3 driver
    ConfigureGPIO(PORTE, 2, PTE, GPIO_OUTPUT_DIRECTION),   // LED4 driver
    ConfigureGPIO(PORTE, 6, PTE, GPIO_OUTPUT_DIRECTION),   // LED5 driver
    ConfigureGPIO(PORTC, 3, PTC, GPIO_OUTPUT_DIRECTION),   // LED6 driver
    ConfigureGPIO(PORTC, 6, PTC, GPIO_OUTPUT_DIRECTION),   // LED7 driver
    ConfigureGPIO(PORTB, 15, PTB, GPIO_OUTPUT_DIRECTION),  // LED8 driver
    ConfigureGPIO(PORTB, 16, PTB, GPIO_OUTPUT_DIRECTION),  // LED9 driver
};

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : init_GPIO
// Description     : Initializes the GPIO pins
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void init_GPIO(void)
{
    /****************************GPIO configuration***************************************/

    uint32_t pinCount = sizeof(g_pin_mux_InitConfigArr) / sizeof(pin_settings_config_t);

    Pins_DRV_Init(pinCount, g_pin_mux_InitConfigArr);

    GPIO_HAL_SetPins(PTD, 1 << LED_RED);      // Turn OFF LED RED
    GPIO_HAL_ClearPins(PTD, 1 << LED_GREEN);  // Turn ON LED GREEN (OPTIONAL)
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : set_reset_pin
// Description     : Sets the state of the PT2001 reset pin (HIGH or LOW)
// Return type     : void 
// Argument        : int state
//////////////////////////////////////////////////////////////////////////////////////////
void set_reset_pin(int state)
{
    if (state == HIGH)
    {
        // Set the reset GPIO pin high
        SET_RESETB_HIGH;
        //GPIO_HAL_SetPins(PTA,2);
    }
    else if (state == LOW)
    {
        // Set the reset GPIO pin low
        SET_RESETB_LOW; //CLEAR PTA1
        //  GPIO_HAL_ClearPins(PTA,2);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : set_drven_pin
// Description     : Sets the state of the Drive Enable pin  (HIGH or LOW)
// Return type     : void 
// Argument        : int state
//////////////////////////////////////////////////////////////////////////////////////////
void set_drven_pin(int state)
{
    if (state == HIGH)
    {
        // Set the drive enable GPIO pin high
        SET_DRVEN_HIGH;
    }
    else if (state == LOW)
    {
        // Set the drive enable GPIO pin low
        SET_DRVEN_LOW;
    }
}
