/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * irq.c
 *
 *  Created on: Apr 20, 2016
 *      Author: B52321
 */
#include "irq.h"
#include "fsl_port_hal.h"
#include "functions.h"
#include "gpio.h"
#include "S32K144.h"
#include "pwm.h"
#include "lpspi.h"
#include "stdio.h"

extern char states;
extern int flag_IRQB;
extern int seq_counter;
int seq_counter_SW2 = 0;
int counter_100us = 0;

uint16_t data_EOI;
uint16_t datatx = 0x0;
long usTime;
uint16_t secontTime = 0;
uint8_t first_edge = 0;
bool flag_spi;

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : PORTA_IRQHandler
// Description     : This interrupt is enabled when PT2001 IRQB pin is low
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void PORTA_IRQHandler(void) //IRQB
{
    char ISF_status = PORT_HAL_GetPortIntFlag(PORTA);
    // Off_sequence(); Optional since PT2001 will shutdown output by itself

    if (ISF_status == PTA3)
    {
        //Set LED Red ON
        GPIO_HAL_SetPins(PTD, 1 << LED_GREEN); //clear GREEN LED
        GPIO_HAL_ClearPins(PTD, 1 << LED_RED); //RED LED set

        flag_IRQB = 1;

        INT_SYS_ClearPending(PORTA_IRQn);
        PORT_HAL_ClearPinIntFlagCmd(PORTA, 3);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : PORTC_IRQHandler
// Description     : This interrupt is enabled in case button SW3 or SW2 is pushed on the
//                   K144 kit
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void PORTC_IRQHandler(void) //Button SW3 and SW2
{
    int ISF_status = PORT_HAL_GetPortIntFlag(PORTC);

    Off_sequence();
    if (ISF_status == PTC13) //SW3
    {
        states = sequence_1; //sequence 1
        seq_counter++;

        if (seq_counter > 1)
        {
            seq_counter = 0;
        }
        PORT_HAL_ClearPinIntFlagCmd(PORTC, 13);
    }

    if (ISF_status == PTC12)  //SW2
    {
            states = sequence_2; //sequence 2
            seq_counter_SW2++;
        if (seq_counter_SW2 > 3)
        {
        	seq_counter_SW2 = 0;
        }

        PORT_HAL_ClearPinIntFlagCmd(PORTC, 12);
    }

    INT_SYS_ClearPending(PORTC_IRQn);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : PORTD_IRQHandler
// Description     : This interrupt is enabled when FLAG1 pin is going high meaning end of
//                   injection is detected.
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void PORTD_IRQHandler(void)
{
    int ISF_status = PORT_HAL_GetPortIntFlag(PORTD);
    // Off_sequence(); Optional since PT2001 will shutdown output by itself

    if (ISF_status == Flag1)
    {
        first_edge++;
        if (first_edge == 1) //falling edge
        {
            FTM1->CNT = 0;
            counter_100us = 0;
        }

        if (first_edge == 2) //rising edge
        {
            //	GPIO_HAL_TogglePins(PTD, 1 << LED_GREEN);
            usTime = 0;
            usTime = FTM_HAL_GetCounter(FTM1);
            //usTime = FTM1->CNT;  //signal 10 KHZ
            usTime = ((usTime * 100) / 598 /*596*/);
            counter_100us += FTM_HAL_HasTimerOverflowed(FTM1);
            counter_100us = (counter_100us * 100);
            usTime = counter_100us + usTime;
            first_edge = 0;
            counter_100us = 0;
            FTM1->CNT = 0;
        }

        flag_spi = 1;
        PORT_HAL_ClearPinIntFlagCmd(PORTD, 8);
        INT_SYS_ClearPending(PORTD_IRQn);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : FTM1_IRQHandler 
// Description     : Every 100us enter on this function, to count the time between edges.
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void FTM1_IRQHandler(void)
{
    counter_100us++;
    if (counter_100us == 10)//10ms reset
    {
        counter_100us = 0;
    }

    FTM_HAL_ClearTimerOverflow(FTM1);
    INT_SYS_ClearPending(FTM1_IRQn); //@ROD
}

