/*******************************************************************************
* Example Code
*
* Copyright(C) 2020 NXP Semiconductors
* NXP Semiconductors Confidential and Proprietary
*
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* NXP products.  This software is supplied "AS IS" without any warranties
* of any kind, and NXP Semiconductors and its licensor disclaim any and
* all warranties, express or implied, including all implied warranties of
* merchantability, fitness for a particular purpose and non-infringement of
* intellectual property rights.  NXP Semiconductors assumes no responsibility
* or liability for the use of the software, conveys no license or rights
* under any patent, copyright, mask work right, or any other intellectual
* property rights in or to any products. NXP Semiconductors reserves the
* right to make changes in the software without notification. NXP
* Semiconductors also makes no representation or warranty that such
* application will be suitable for the specified use without further testing
* or modification.
*
* IN NO EVENT WILL NXP SEMICONDUCTORS BE LIABLE, WHETHER IN CONTRACT, 
* TORT, OR OTHERWISE, FOR ANY INCIDENTAL, SPECIAL, INDIRECT, CONSEQUENTIAL 
* OR PUNITIVE DAMAGES, INCLUDING, BUT NOT LIMITED TO, DAMAGES FOR ANY 
* LOSS OF USE, LOSS OF TIME, INCONVENIENCE, COMMERCIAL LOSS, OR LOST 
* PROFITS, SAVINGS, OR REVENUES, TO THE FULL EXTENT SUCH MAY BE DISCLAIMED  
* BY LAW. NXP SEMICONDUCTOR’S TOTAL LIABILITY FOR ALL COSTS, DAMAGES, 
* CLAIMS, OR LOSSES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE IS LIMITED TO THE AGGREGATE AMOUNT PAID BY YOU TO NXP SEMICONDUCTORS
* IN CONNECTION WITH THE SOFTWARE TO WHICH LOSSES OR DAMAGES ARE CLAIMED.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors' and its
* licensor's relevant copyrights in the software, without fee, provided
* that it is used in conjunction with NXP Semiconductors devices.  This
* copyright, permission, and disclaimer notice must appear in all copies
* of this code.
*******************************************************************************/
/*
 * pwm.c
 *
 *  Created on: Apr 18, 2016
 *      Author: B52321
 */

#include "pwm.h"
#include "derivative.h"
#include "fsl_interrupt_manager.h"


//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : PWM_1MHz
// Description     : Initialize the 1MHz PWM signal
// Return type     : void
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void PWM_1MHz(void)
{
    ftm_user_config_t  flexTimer2_InitConfig =
    {
        {
            true,              /* Software trigger state */
            false,             /* Hardware trigger 1 state */
            false,             /* Hardware trigger 2 state */
            false,             /* Hardware trigger 3 state */
            false,             /* Max loading point state */
            false,             /* Min loading point state */
            FTM_SYSTEM_CLOCK,  /* Update mode for INVCTRL register */
            FTM_SYSTEM_CLOCK,  /* Update mode for SWOCTRL register */
            FTM_SYSTEM_CLOCK,  /* Update mode for OUTMASK register */
            FTM_SYSTEM_CLOCK,  /* Update mode for CNTIN register */
        },
        FTM_CLOCK_DIVID_BY_2,        /* FTM clock prescaler */
        FTM_CLOCK_SOURCE_SYSTEMCLK,  /* FTM clock source */
        FTM_BDM_MODE_11,	 	     /* FTM debug mode */
        0U,                          /* Overflow ratio */
        false,                       /* Write protection */
        false                        /* Interrupt state */
    };
    FTM_DRV_Init(flexTimer2_INSTANCE, &flexTimer2_InitConfig);

    ftm_independent_ch_param_t flexTimer1_IndependentChannelsConfig[1] =
    {
        {
            5U,                /* hwChannelId */
            FTM_POLARITY_LOW,  /* Edge mode */
            17000,             /* Duty cycle percent 0-0x8000 */ //32894U
        }
    };

    ftm_combined_ch_param_t flexTimer1_CombinedChannelsConfig[0] =
    {
    };

    /* Fault configuration structure for FTM0 */
    ftm_pwm_fault_param_t flexTimer1_FaultConfig =
    {
        false,
        false,
        0U, /* Fault filter value */
        FTM_FAULT_CONTROL_DISABLED,
        {
            {
                true,  /* Fault channel state (Enabled/Disabled) */
                false, /* Fault channel filter state (Enabled/Disabled) */
                FTM_POLARITY_HIGH, /* Fault channel state (Enabled/Disabled) */
            },
            {
                false, /* Fault Channel state (Enabled/Disabled) */
                false, /* Fault channel filter state (Enabled/Disabled) */
                FTM_POLARITY_LOW  /* Fault Pin Polarity */
            },
            {
                false, /* Fault Channel state (Enabled/Disabled) */
                false, /* Fault channel filter state (Enabled/Disabled) */
                FTM_POLARITY_LOW  /* Fault Pin Polarity */
            },
            {
                false, /* Fault Channel state (Enabled/Disabled) */
                false, /* Fault channel filter state (Enabled/Disabled) */
                FTM_POLARITY_LOW  /* Fault Pin Polarity */
            }
        }
    };

    /*PWM configuration for flexTimer1 */
    ftm_pwm_param_t flexTimer2_PwmConfig =
    {
        1U,                                    /* Number of independent PWM channels */
        0U,                                    /* Number of combined PWM channels */
        FTM_MODE_OUTPUT_COMPARE,               /* PWM mode */
        0U,                                    /* Dead time value */
        FTM_DEADTIME_DIVID_BY_1,               /* Dead time clock divider */
        1000000U,                              /* PWM frequency */ //1MHZ
        &flexTimer1_IndependentChannelsConfig, /* Independent PWM channels config structure */
        &flexTimer1_CombinedChannelsConfig,    /* Combined PWM channels config structure */
        &flexTimer1_FaultConfig                /* PWM fault config structure */
    };

    FTM_DRV_InitPwm(flexTimer2_INSTANCE, &flexTimer2_PwmConfig);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Function name   : InputCaptureTimer_init
// Description     : Initializes the input capture timer.
// Return type     : void 
// Argument        : void
//////////////////////////////////////////////////////////////////////////////////////////
void InputCaptureTimer_init(void)
{
    ftm_user_config_t  flexTimer1_InitConfig =
    {
        {
            true,               /* Software trigger state */
            false,              /* Hardware trigger 1 state */
            false,              /* Hardware trigger 2 state */
            false,              /* Hardware trigger 3 state */
            false,              /* Max loading point state */
            false,              /* Min loading point state */
            FTM_SYSTEM_CLOCK,   /* Update mode for INVCTRL register */
            FTM_SYSTEM_CLOCK,   /* Update mode for SWOCTRL register */
            FTM_SYSTEM_CLOCK,   /* Update mode for OUTMASK register */
            FTM_SYSTEM_CLOCK,   /* Update mode for CNTIN register */
        },
        FTM_CLOCK_DIVID_BY_4,           /* FTM clock prescaler */
        FTM_CLOCK_SOURCE_SYSTEMCLK,     /* FTM clock source */
        FTM_BDM_MODE_11,                /* FTM debug mode */
        0U,                             /* Overflow ratio */
        false,                          /* Write protection */
        true,                           /* Interrupt state */
    };

    FTM_DRV_Init(flexTimer1_INSTANCE, &flexTimer1_InitConfig);

    ftm_input_ch_param_t  inputChConfig[1] =
    {
        {
            .hwChannelId = 4,
            .InputMode = FTM_NO_OPERATION,
            .EdgeAlignement = FTM_BOTH_EDGES,
            .MeasurementType = FTM_PERIOD_MEASUREMENT,
            .FilterValue = 0,
            .FilterEn = false,
            .ContinuousModeEn = false,
        }
    };

    ftm_input_param_t flexTimer1_IntCapConfig =
    {
        .nNumChannels = 1,
        .nMaxCountValue = 598, //5KHZ 598
        &inputChConfig,
    };

    INT_SYS_SetPriority(PORTD_IRQn, 7);
    INT_SYS_SetPriority(LPSPI0_IRQn, 9);
    FTM_DRV_InitInputCapture(flexTimer1_INSTANCE, &flexTimer1_IntCapConfig);
    FTM_HAL_ClearChnEventFlag(FTM1, 4);
}
